@@ -18,7 +18,12 @@ and :class:`Colorbar`; the :func:`~matplotlib.pyplot.colorbar` function
 is a thin wrapper over :meth:`~matplotlib.figure.Figure.colorbar`.
 
 '''
-from __future__ import print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+from six.moves import xrange, zip
+
 import warnings
 
 import numpy as np
@@ -31,7 +36,6 @@ import matplotlib.colors as colors
 import matplotlib.contour as contour
 import matplotlib.cm as cm
 import matplotlib.gridspec as gridspec
-import matplotlib.lines as lines
 import matplotlib.patches as mpatches
 import matplotlib.path as mpath
 import matplotlib.ticker as ticker
@@ -415,9 +419,12 @@ class ColorbarBase(cm.ScalarMappable):
         ax.set_ylim(*ax.dataLim.intervaly)
         if self.outline is not None:
             self.outline.remove()
-        self.outline = lines.Line2D(
-            xy[:, 0], xy[:, 1], color=mpl.rcParams['axes.edgecolor'],
-            linewidth=mpl.rcParams['axes.linewidth'])
+        self.outline = mpatches.Polygon(
+            xy, edgecolor=mpl.rcParams['axes.edgecolor'],
+            facecolor='none',
+            linewidth=mpl.rcParams['axes.linewidth'],
+            closed=True,
+            zorder=2)
         ax.add_artist(self.outline)
         self.outline.set_clip_box(None)
         self.outline.set_clip_path(None)
@@ -469,9 +476,9 @@ class ColorbarBase(cm.ScalarMappable):
         # Using the non-array form of these line segments is much
         # simpler than making them into arrays.
         if self.orientation == 'vertical':
-            return [zip(X[i], Y[i]) for i in xrange(1, N - 1)]
+            return [list(zip(X[i], Y[i])) for i in xrange(1, N - 1)]
         else:
-            return [zip(Y[i], X[i]) for i in xrange(1, N - 1)]
+            return [list(zip(Y[i], X[i])) for i in xrange(1, N - 1)]
 
     def _add_solids(self, X, Y, C):
         '''
@@ -529,9 +536,9 @@ class ColorbarBase(cm.ScalarMappable):
         x = np.array([0.0, 1.0])
         X, Y = np.meshgrid(x, y)
         if self.orientation == 'vertical':
-            xy = [zip(X[i], Y[i]) for i in xrange(N)]
+            xy = [list(zip(X[i], Y[i])) for i in xrange(N)]
         else:
-            xy = [zip(Y[i], X[i]) for i in xrange(N)]
+            xy = [list(zip(Y[i], X[i])) for i in xrange(N)]
         col = collections.LineCollection(xy, linewidths=linewidths)
 
         if erase and self.lines:
@@ -544,8 +551,8 @@ class ColorbarBase(cm.ScalarMappable):
 
     def _ticker(self):
         '''
-        Return two sequences: ticks (colorbar data locations)
-        and ticklabels (strings).
+        Return the sequence of ticks (colorbar data locations),
+        ticklabels (strings), and the corresponding offset string.
         '''
         locator = self.locator
         formatter = self.formatter
@@ -646,6 +653,9 @@ class ColorbarBase(cm.ScalarMappable):
             self._values = v
             return
         else:
+            self.norm.vmin, self.norm.vmax = mtrans.nonsingular(self.norm.vmin,
+                                                                self.norm.vmax,
+                                                                expander=0.1)
             if not self.norm.scaled():
                 self.norm.vmin = 0
                 self.norm.vmax = 1
@@ -694,7 +704,7 @@ class ColorbarBase(cm.ScalarMappable):
         '''
         # Set the default value.
         extendlength = np.array([default, default])
-        if isinstance(frac, str):
+        if isinstance(frac, six.string_types):
             if frac.lower() == 'auto':
                 # Use the provided values when 'auto' is required.
                 extendlength[0] = automin
@@ -810,29 +820,34 @@ class ColorbarBase(cm.ScalarMappable):
             # as to make the interpolation more accurate.
             b = self.norm(self._boundaries, clip=False).filled()
             xn = self.norm(x, clip=False).filled()
+
         # The rest is linear interpolation with extrapolation at ends.
-        y = self._y
-        N = len(b)
         ii = np.searchsorted(b, xn)
         i0 = ii - 1
-        itop = (ii == N)
+        itop = (ii == len(b))
         ibot = (ii == 0)
         i0[itop] -= 1
         ii[itop] -= 1
         i0[ibot] += 1
         ii[ibot] += 1
 
-        #db = b[ii] - b[i0]
         db = np.take(b, ii) - np.take(b, i0)
-        #dy = y[ii] - y[i0]
+        y = self._y
         dy = np.take(y, ii) - np.take(y, i0)
         z = np.take(y, i0) + (xn - np.take(b, i0)) * dy / db
-
         return z
 
     def set_alpha(self, alpha):
         self.alpha = alpha
 
+    def remove(self):
+        """
+        Remove this colorbar from the figure
+        """
+
+        fig = self.ax.figure
+        fig.delaxes(self.ax)
+
 
 class Colorbar(ColorbarBase):
     """
@@ -960,6 +975,33 @@ class Colorbar(ColorbarBase):
         # be recalculating everything if there was a simple alpha
         # change.
 
+    def remove(self):
+        """
+        Remove this colorbar from the figure.  If the colorbar was created with
+        ``use_gridspec=True`` then restore the gridspec to its previous value.
+        """
+
+        ColorbarBase.remove(self)
+        self.mappable.callbacksSM.disconnect(self.mappable.colorbar_cid)
+        self.mappable.colorbar = None
+        self.mappable.colorbar_cid = None
+
+        try:
+            ax = self.mappable.axes
+        except AttributeError:
+            return
+
+        try:
+            gs = ax.get_subplotspec().get_gridspec()
+            subplotspec = gs.get_topmost_subplotspec()
+        except AttributeError:
+            # use_gridspec was False
+            pos = ax.get_position(original=True)
+            ax.set_position(pos)
+        else:
+            # use_gridspec was True
+            ax.set_subplotspec(subplotspec)
+
 
 @docstring.Substitution(make_axes_kw_doc)
 def make_axes(parents, location=None, orientation=None, fraction=0.15,
@@ -1273,7 +1315,8 @@ def colorbar_factory(cax, mappable, **kwargs):
     else:
         cb = Colorbar(cax, mappable, **kwargs)
 
-    mappable.callbacksSM.connect('changed', cb.on_mappable_changed)
+    cid = mappable.callbacksSM.connect('changed', cb.on_mappable_changed)
     mappable.colorbar = cb
+    mappable.colorbar_cid = cid
 
     return cb
