@@ -1,10 +1,14 @@
 """
 Tests specific to the collections module.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
 
 from nose.tools import assert_equal
 import numpy as np
-from numpy.testing import assert_array_equal
+from numpy.testing import assert_array_equal, assert_array_almost_equal
 
 import matplotlib.pyplot as plt
 import matplotlib.collections as mcollections
@@ -229,8 +233,8 @@ def test__EventCollection__switch_orientation():
     splt.set_xlim(0, 2)
 
 
-@image_comparison(baseline_images=
-                  ['EventCollection_plot__switch_orientation__2x'])
+@image_comparison(
+    baseline_images=['EventCollection_plot__switch_orientation__2x'])
 def test__EventCollection__switch_orientation_2x():
     '''
     check to make sure calling switch_orientation twice sets the
@@ -396,7 +400,117 @@ def test_null_collection_datalim():
     col = mcollections.PathCollection([])
     col_data_lim = col.get_datalim(mtransforms.IdentityTransform())
     assert_array_equal(col_data_lim.get_points(),
-                       mtransforms.Bbox([[0, 0], [0, 0]]).get_points())
+                       mtransforms.Bbox.null().get_points())
+
+
+@cleanup
+def test_add_collection():
+    # Test if data limits are unchanged by adding an empty collection.
+    # Github issue #1490, pull #1497.
+    ax = plt.axes()
+    plt.figure()
+    ax2 = plt.axes()
+    coll = ax2.scatter([0, 1], [0, 1])
+    ax.add_collection(coll)
+    bounds = ax.dataLim.bounds
+    coll = ax2.scatter([], [])
+    ax.add_collection(coll)
+    assert_equal(ax.dataLim.bounds, bounds)
+
+
+@cleanup
+def test_quiver_limits():
+    ax = plt.axes()
+    x, y = np.arange(8), np.arange(10)
+    data = u = v = np.linspace(0, 10, 80).reshape(10, 8)
+    q = plt.quiver(x, y, u, v)
+    assert_equal(q.get_datalim(ax.transData).bounds, (0., 0., 7., 9.))
+
+    plt.figure()
+    ax = plt.axes()
+    x = np.linspace(-5, 10, 20)
+    y = np.linspace(-2, 4, 10)
+    y, x = np.meshgrid(y, x)
+    trans = mtransforms.Affine2D().translate(25, 32) + ax.transData
+    plt.quiver(x, y, np.sin(x), np.cos(y), transform=trans)
+    assert_equal(ax.dataLim.bounds, (20.0, 30.0, 15.0, 6.0))
+
+
+@cleanup
+def test_barb_limits():
+    ax = plt.axes()
+    x = np.linspace(-5, 10, 20)
+    y = np.linspace(-2, 4, 10)
+    y, x = np.meshgrid(y, x)
+    trans = mtransforms.Affine2D().translate(25, 32) + ax.transData
+    plt.barbs(x, y, np.sin(x), np.cos(y), transform=trans)
+    # The calculated bounds are approximately the bounds of the original data,
+    # this is because the entire path is taken into account when updating the
+    # datalim.
+    assert_array_almost_equal(ax.dataLim.bounds, (20, 30, 15, 6),
+                              decimal=1)
+
+
+@image_comparison(baseline_images=['EllipseCollection_test_image'],
+                  extensions=['png'],
+                  remove_text=True)
+def test_EllipseCollection():
+    # Test basic functionality
+    fig, ax = plt.subplots()
+    x = np.arange(4)
+    y = np.arange(3)
+    X, Y = np.meshgrid(x, y)
+    XY = np.vstack((X.ravel(), Y.ravel())).T
+
+    ww = X/float(x[-1])
+    hh = Y/float(y[-1])
+    aa = np.ones_like(ww) * 20  # first axis is 20 degrees CCW from x axis
+
+    ec = mcollections.EllipseCollection(ww, hh, aa,
+                                        units='x',
+                                        offsets=XY,
+                                        transOffset=ax.transData,
+                                        facecolors='none')
+    ax.add_collection(ec)
+    ax.autoscale_view()
+
+
+@image_comparison(baseline_images=['polycollection_close'],
+                  extensions=['png'], remove_text=True)
+def test_polycollection_close():
+    from mpl_toolkits.mplot3d import Axes3D
+
+    vertsQuad = [
+        [[0., 0.], [0., 1.], [1., 1.], [1., 0.]],
+        [[0., 1.], [2., 3.], [2., 2.], [1., 1.]],
+        [[2., 2.], [2., 3.], [4., 1.], [3., 1.]],
+        [[3., 0.], [3., 1.], [4., 1.], [4., 0.]]]
+
+    fig = plt.figure()
+    ax = Axes3D(fig)
+
+    colors = ['r', 'g', 'b', 'y', 'k']
+    zpos = list(range(5))
+
+    poly = mcollections.PolyCollection(
+        vertsQuad * len(zpos), linewidth=0.25)
+    poly.set_alpha(0.7)
+
+    ## need to have a z-value for *each* polygon = element!
+    zs = []
+    cs = []
+    for z, c in zip(zpos, colors):
+        zs.extend([z] * len(vertsQuad))
+        cs.extend([c] * len(vertsQuad))
+
+    poly.set_color(cs)
+
+    ax.add_collection3d(poly, zs=zs, zdir='y')
+
+    ## axis limit settings:
+    ax.set_xlim3d(0, 4)
+    ax.set_zlim3d(0, 3)
+    ax.set_ylim3d(0, 4)
 
 
 if __name__ == '__main__':
