@@ -1,37 +1,38 @@
 #!/usr/bin/env python
+
 """
 Show how to make date plots in matplotlib using date tick locators and
 formatters.  See major_minor_demo1.py for more information on
 controlling major and minor ticks
 """
+from __future__ import print_function
 import datetime
-from pylab import figure, show
-from matplotlib.dates import MONDAY, SATURDAY
-from matplotlib.finance import quotes_historical_yahoo
+import matplotlib.pyplot as plt
+from matplotlib.dates import MONDAY
+from matplotlib.finance import quotes_historical_yahoo_ochl
 from matplotlib.dates import MonthLocator, WeekdayLocator, DateFormatter
 
 
-date1 = datetime.date( 2002, 1, 5 )
-date2 = datetime.date( 2003, 12, 1 )
+date1 = datetime.date(2002, 1, 5)
+date2 = datetime.date(2003, 12, 1)
 
 # every monday
-mondays   = WeekdayLocator(MONDAY)
+mondays = WeekdayLocator(MONDAY)
 
 # every 3rd month
-months    = MonthLocator(range(1,13), bymonthday=1, interval=3)
+months = MonthLocator(range(1, 13), bymonthday=1, interval=3)
 monthsFmt = DateFormatter("%b '%y")
 
 
-quotes = quotes_historical_yahoo('INTC', date1, date2)
+quotes = quotes_historical_yahoo_ochl('INTC', date1, date2)
 if len(quotes) == 0:
-    print 'Found no quotes'
+    print ('Found no quotes')
     raise SystemExit
 
 dates = [q[0] for q in quotes]
 opens = [q[1] for q in quotes]
 
-fig = figure()
-ax = fig.add_subplot(111)
+fig, ax = plt.subplots()
 ax.plot_date(dates, opens, '-')
 ax.xaxis.set_major_locator(months)
 ax.xaxis.set_major_formatter(monthsFmt)
@@ -43,4 +44,4 @@ ax.grid(True)
 
 fig.autofmt_xdate()
 
-show()
+plt.show()
