@@ -10,9 +10,13 @@ import os
 import re
 import subprocess
 import sys
+import warnings
 from textwrap import fill
 
 
+PY3 = (sys.version_info[0] >= 3)
+
+
 try:
     from subprocess import check_output
 except ImportError:
@@ -46,10 +50,10 @@ if sys.platform != 'win32':
         from subprocess import getstatusoutput
 
 
-if sys.version_info[0] < 3:
-    import ConfigParser as configparser
-else:
+if PY3:
     import configparser
+else:
+    import ConfigParser as configparser
 
 
 # matplotlib build options, which can be altered using setup.cfg
@@ -286,7 +290,7 @@ class PkgConfig(object):
                 for token in output.split():
                     attr = flag_map.get(token[:2])
                     if attr is not None:
-                        getattr(ext, attr).append(token[2:])
+                        getattr(ext, attr).insert(0, token[2:])
 
         if use_defaults:
             basedirs = get_base_dirs()
@@ -446,14 +450,15 @@ class OptionalPackage(SetupPackage):
     force = False
     config_category = "packages"
 
-    def get_config(self):
+    @classmethod
+    def get_config(cls):
         """
         Look at `setup.cfg` and return one of ["auto", True, False] indicating
         if the package is at default state ("auto"), forced by the user (True)
         or opted-out (False).
         """
         try:
-            return config.getboolean(self.config_category, self.name)
+            return config.getboolean(cls.config_category, cls.name)
         except:
             return "auto"
 
@@ -541,10 +546,12 @@ class Matplotlib(SetupPackage):
         return [
             'matplotlib',
             'matplotlib.backends',
-            'matplotlib.backends.qt4_editor',
+            'matplotlib.backends.qt_editor',
             'matplotlib.compat',
             'matplotlib.projections',
+            'matplotlib.axes',
             'matplotlib.sphinxext',
+            'matplotlib.style',
             'matplotlib.testing',
             'matplotlib.testing.jpl_units',
             'matplotlib.tri',
@@ -577,7 +584,8 @@ class Matplotlib(SetupPackage):
                 'backends/web_backend/jquery/css/themes/base/*.*',
                 'backends/web_backend/jquery/css/themes/base/images/*',
                 'backends/web_backend/css/*.*',
-                'backends/Matplotlib.nib/*'
+                'backends/Matplotlib.nib/*',
+                'mpl-data/stylelib/*.mplstyle',
              ]}
 
 
@@ -615,23 +623,40 @@ class Toolkits(OptionalPackage):
 
 class Tests(OptionalPackage):
     name = "tests"
+    nose_min_version = '0.11.1'
 
     def check(self):
         super(Tests, self).check()
 
+        msgs = []
+        msg_template = ('{package} is required to run the matplotlib test '
+                        'suite.  pip/easy_install may attempt to install it '
+                        'after matplotlib.')
+
+
+        bad_nose = msg_template.format(
+            package='nose %s or later' % self.nose_min_version
+        )
         try:
             import nose
+            if is_min_version(nose.__version__, self.nose_min_version):
+                msgs += ['using nose version %s' % nose.__version__]
+            else:
+                msgs += [bad_nose]
         except ImportError:
-            return (
-                "nose 0.11.1 or later is required to run the "
-                "matplotlib test suite")
+            msgs += [bad_nose]
 
-        if nose.__versioninfo__ < (0, 11, 1):
-            return (
-                "nose 0.11.1 or later is required to run the "
-                "matplotlib test suite")
 
-        return 'using nose version %s' % nose.__version__
+        if sys.version_info >= (3, 3):
+            msgs += ['using unittest.mock']
+        else:
+            try:
+                import mock
+                msgs += ['using mock %s' % mock.__version__]
+            except ImportError:
+                msgs += [msg_template.format(package='mock')]
+
+        return ' / '.join(msgs)
 
     def get_packages(self):
         return [
@@ -652,7 +677,42 @@ class Tests(OptionalPackage):
             ]}
 
     def get_install_requires(self):
-        return ['nose']
+        requires = ['nose>=%s' % self.nose_min_version]
+        if not sys.version_info >= (3, 3):
+            requires += ['mock']
+        return requires
+
+class Toolkits_Tests(Tests):
+    name = "toolkits_tests"
+
+    def check_requirements(self):
+        conf = self.get_config()
+        toolkits_conf = Toolkits.get_config()
+        tests_conf = Tests.get_config()
+
+        if conf is True:
+            Tests.force = True
+            Toolkits.force = True
+        elif conf == "auto" and not (toolkits_conf and tests_conf):
+            # Only auto-install if both toolkits and tests are set
+            # to be installed
+            raise CheckFailed("toolkits_tests needs 'toolkits' and 'tests'")
+        return ""
+
+    def get_packages(self):
+        return [
+            'mpl_toolkits.tests',
+            ]
+
+    def get_package_data(self):
+        baseline_images = [
+            'tests/baseline_images/%s/*' % x
+            for x in os.listdir('lib/mpl_toolkits/tests/baseline_images')]
+
+        return {'mpl_toolkits': baseline_images}
+
+    def get_namespace_packages(self):
+        return ['mpl_toolkits']
 
 
 class DelayedExtension(Extension, object):
@@ -758,17 +818,17 @@ class Numpy(SetupPackage):
         ext.add_hook('include_dirs', self.include_dirs_hook)
 
     def get_setup_requires(self):
-        return ['numpy>=1.5']
+        return ['numpy>=1.6']
 
     def get_install_requires(self):
-        return ['numpy>=1.5']
+        return ['numpy>=1.6']
 
 
 class CXX(SetupPackage):
     name = 'pycxx'
 
     def check(self):
-        if sys.version_info[0] >= 3:
+        if PY3:
             # There is no version of PyCXX in the wild that will work
             # with Python 3.x
             self.__class__.found_external = False
@@ -777,7 +837,7 @@ class CXX(SetupPackage):
 
         self.__class__.found_external = True
         old_stdout = sys.stdout
-        if sys.version_info[0] >= 3:
+        if PY3:
             sys.stdout = io.StringIO()
         else:
             sys.stdout = io.BytesIO()
@@ -820,10 +880,11 @@ class CXX(SetupPackage):
                  'cxxextensions.c']])
             pkg_config.setup_extension(ext, 'PyCXX')
         else:
-            ext.sources.extend(glob.glob('CXX/*.cxx'))
-            ext.sources.extend(glob.glob('CXX/*.c'))
+            ext.include_dirs.append('extern')
+            ext.sources.extend(glob.glob('extern/CXX/*.cxx'))
+            ext.sources.extend(glob.glob('extern/CXX/*.c'))
         ext.define_macros.append(('PYCXX_ISO_CPP_LIB', '1'))
-        if sys.version_info[0] >= 3:
+        if PY3:
             ext.define_macros.append(('PYCXX_PYTHON_2TO3', '1'))
         if not (sys.platform == 'win32' and win32_compiler == 'msvc'):
             ext.libraries.append('stdc++')
@@ -846,7 +907,7 @@ class LibAgg(SetupPackage):
         if self.found_external:
             pkg_config.setup_extension(ext, 'libagg')
         else:
-            ext.include_dirs.append('agg24/include')
+            ext.include_dirs.append('extern/agg24/include')
             agg_sources = [
                 'agg_bezier_arc.cpp',
                 'agg_curves.cpp',
@@ -858,7 +919,7 @@ class LibAgg(SetupPackage):
                 'agg_vpgen_segmentator.cpp'
                 ]
             ext.sources.extend(
-                os.path.join('agg24', 'src', x) for x in agg_sources)
+                os.path.join('extern', 'agg24', 'src', x) for x in agg_sources)
 
 
 class FreeType(SetupPackage):
@@ -868,16 +929,43 @@ class FreeType(SetupPackage):
         if sys.platform == 'win32':
             return "Unknown version"
 
-        status, output = getstatusoutput("freetype-config --version")
+        status, output = getstatusoutput("freetype-config --ftversion")
         if status == 0:
             version = output
         else:
             version = None
 
+        # Early versions of freetype grep badly inside freetype-config,
+        # so catch those cases. (tested with 2.5.3).
+        if 'No such file or directory\ngrep:' in version:
+            version = self.version_from_header()
+
         return self._check_for_pkg_config(
             'freetype2', 'ft2build.h',
             min_version='2.4', version=version)
 
+    def version_from_header(self):
+        version = 'Failed to identify version.'
+        ext = self.get_extension()
+        if ext is None:
+            return version
+        # Return the first version found in the include dirs.
+        for include_dir in ext.include_dirs:
+            header_fname = os.path.join(include_dir, 'freetype.h')
+            if os.path.exists(header_fname):
+                major, minor, patch = 0, 0, 0
+                with open(header_fname, 'r') as fh:
+                    for line in fh:
+                        if line.startswith('#define FREETYPE_'):
+                            value = line.rsplit(' ', 1)[1].strip()
+                            if 'MAJOR' in line:
+                                major = value
+                            elif 'MINOR' in line:
+                                minor = value
+                            else:
+                                patch = value
+                return '.'.join([major, minor, patch])
+
     def add_flags(self, ext):
         pkg_config.setup_extension(
             ext, 'freetype2',
@@ -889,6 +977,12 @@ class FreeType(SetupPackage):
             default_libraries=['freetype', 'z'],
             alt_exec='freetype-config')
 
+    def get_extension(self):
+        if sys.platform == 'win32':
+            return None
+        ext = make_extension('freetype2', [])
+        self.add_flags(ext)
+        return ext
 
 class FT2Font(SetupPackage):
     name = 'ft2font'
@@ -929,19 +1023,50 @@ class Png(SetupPackage):
         return ext
 
 
+class Qhull(SetupPackage):
+    name = "qhull"
+
+    def check(self):
+        self.__class__.found_external = True
+        try:
+            return self._check_for_pkg_config(
+                'qhull', 'qhull/qhull_a.h', min_version='2003.1')
+        except CheckFailed as e:
+            self.__class__.found_pkgconfig = False
+            # Qhull may not be in the pkg-config system but may still be
+            # present on this system, so check if the header files can be
+            # found.
+            include_dirs = [
+                os.path.join(x, 'include', 'qhull') for x in get_base_dirs()]
+            if has_include_file(include_dirs, 'qhull_a.h'):
+                return 'Using system Qhull (version unknown, no pkg-config info)'
+            else:
+                self.__class__.found_external = False
+                return str(e) + ' Using local copy.'
+
+    def add_flags(self, ext):
+        if self.found_external:
+            pkg_config.setup_extension(ext, 'qhull',
+                                       default_libraries=['qhull'])
+        else:
+            ext.include_dirs.append('extern')
+            ext.sources.extend(glob.glob('extern/qhull/*.c'))
+
+
 class TTConv(SetupPackage):
     name = "ttconv"
 
     def get_extension(self):
         sources = [
             'src/_ttconv.cpp',
-            'ttconv/pprdrv_tt.cpp',
-            'ttconv/pprdrv_tt2.cpp',
-            'ttconv/ttutil.cpp'
+            'extern/ttconv/pprdrv_tt.cpp',
+            'extern/ttconv/pprdrv_tt2.cpp',
+            'extern/ttconv/ttutil.cpp'
             ]
         ext = make_extension('matplotlib.ttconv', sources)
         Numpy().add_flags(ext)
         CXX().add_flags(ext)
+        ext.include_dirs.append('extern')
         return ext
 
 
@@ -1003,6 +1128,18 @@ class Delaunay(SetupPackage):
         return ext
 
 
+class QhullWrap(SetupPackage):
+    name = "qhull_wrap"
+
+    def get_extension(self):
+        sources = ['src/qhull_wrap.c']
+        ext = make_extension('matplotlib._qhull', sources,
+                             define_macros=[('MPL_DEVNULL', os.devnull)])
+        Numpy().add_flags(ext)
+        Qhull().add_flags(ext)
+        return ext
+
+
 class Tri(SetupPackage):
     name = "tri"
 
@@ -1017,6 +1154,28 @@ class Tri(SetupPackage):
         return ext
 
 
+class Six(SetupPackage):
+    name = "six"
+    min_version = "1.3"
+
+    def check(self):
+        try:
+            import six
+        except ImportError:
+            return (
+                "six was not found.")
+
+        if not is_min_version(six.__version__, self.min_version):
+            raise CheckFailed(
+                "Requires six %s or later.  Found %s." %
+                (self.min_version, six.__version__))
+
+        return "using six version %s" % six.__version__
+
+    def get_install_requires(self):
+        return ['six>={0}'.format(self.min_version)]
+
+
 class Dateutil(SetupPackage):
     name = "dateutil"
 
@@ -1049,7 +1208,7 @@ class Dateutil(SetupPackage):
         return [dateutil]
 
 
-class Tornado(SetupPackage):
+class Tornado(OptionalPackage):
     name = "tornado"
 
     def check(self):
@@ -1063,9 +1222,6 @@ class Tornado(SetupPackage):
 
         return "using tornado version %s" % tornado.version
 
-    def get_install_requires(self):
-        return ['tornado']
-
 
 class Pyparsing(SetupPackage):
     name = "pyparsing"
@@ -1077,7 +1233,7 @@ class Pyparsing(SetupPackage):
             f = pyparsing.Forward()
             f <<= pyparsing.Literal('a')
             return f is not None
-        except:
+        except (ImportError, TypeError):
             return False
 
     def check(self):
@@ -1135,10 +1291,10 @@ class BackendTkAgg(OptionalBackendPackage):
 
     def check_requirements(self):
         try:
-            if sys.version_info[0] < 3:
-                import Tkinter
-            else:
+            if PY3:
                 import tkinter as Tkinter
+            else:
+                import Tkinter
         except ImportError:
             raise CheckFailed('TKAgg requires Tkinter.')
         except RuntimeError:
@@ -1187,10 +1343,10 @@ class BackendTkAgg(OptionalBackendPackage):
             return self.tcl_tk_cache
 
         # By this point, we already know that Tkinter imports correctly
-        if sys.version_info[0] < 3:
-            import Tkinter
-        else:
+        if PY3:
             import tkinter as Tkinter
+        else:
+            import Tkinter
         tcl_lib_dir = ''
         tk_lib_dir = ''
         # First try to open a Tk window (requires a running X server)
@@ -1228,10 +1384,10 @@ class BackendTkAgg(OptionalBackendPackage):
 
     def parse_tcl_config(self, tcl_lib_dir, tk_lib_dir):
         try:
-            if sys.version_info[0] < 3:
-                import Tkinter
-            else:
+            if PY3:
                 import tkinter as Tkinter
+            else:
+                import Tkinter
         except ImportError:
             return None
 
@@ -1329,8 +1485,12 @@ class BackendTkAgg(OptionalBackendPackage):
     def add_flags(self, ext):
         if sys.platform == 'win32':
             major, minor1, minor2, s, tmp = sys.version_info
-            ext.include_dirs.extend(['win32_static/include/tcl85'])
-            ext.libraries.extend(['tk85', 'tcl85'])
+            if sys.version_info[0:2] < (3, 4):
+                ext.include_dirs.extend(['win32_static/include/tcl85'])
+                ext.libraries.extend(['tk85', 'tcl85'])
+            else:
+                ext.include_dirs.extend(['win32_static/include/tcl86'])
+                ext.libraries.extend(['tk86t', 'tcl86t'])
             ext.library_dirs.extend([os.path.join(sys.prefix, 'dlls')])
 
         elif sys.platform == 'darwin':
@@ -1578,9 +1738,6 @@ class BackendGtk3Agg(OptionalBackendPackage):
         if 'TRAVIS' in os.environ:
             raise CheckFailed("Can't build with Travis")
 
-        if sys.version_info[0] >= 3:
-            raise CheckFailed("gtk3agg backend does not work on Python 3")
-
         # This check needs to be performed out-of-process, because
         # importing gi and then importing regular old pygtk afterward
         # segfaults the interpreter.
@@ -1609,9 +1766,12 @@ class BackendGtk3Agg(OptionalBackendPackage):
 
 def backend_gtk3cairo_internal_check(x):
     try:
-        import cairo
+        import cairocffi
     except ImportError:
-        return (False, "Requires cairo to be installed.")
+        try:
+            import cairo
+        except ImportError:
+            return (False, "Requires cairocffi or pycairo to be installed.")
 
     try:
         import gi
@@ -1752,8 +1912,7 @@ class Windowing(OptionalBackendPackage):
         return ext
 
 
-class BackendQt4(OptionalBackendPackage):
-    name = "qt4agg"
+class BackendQtBase(OptionalBackendPackage):
 
     def convert_qt_version(self, version):
         version = '%x' % version
@@ -1764,36 +1923,108 @@ class BackendQt4(OptionalBackendPackage):
         return '.'.join(temp)
 
     def check_requirements(self):
+        '''
+        If PyQt4/PyQt5 is already imported, importing PyQt5/PyQt4 will fail
+        so we need to test in a subprocess (as for Gtk3).
+        '''
         try:
-            from PyQt4 import pyqtconfig
-        except ImportError:
-            raise CheckFailed("PyQt4 not found")
-        # Import may still be broken for our python
-        try:
-            qtconfig = pyqtconfig.Configuration()
-        except AttributeError:
-            raise CheckFailed('PyQt4 not correctly imported')
+            p = multiprocessing.Pool()
+
+        except:
+            # Can't do multiprocessing, fall back to normal approach ( this will fail if importing both PyQt4 and PyQt5 )
+            try:
+                # Try in-process
+                msg = self.callback(self)
+
+            except RuntimeError:
+                raise CheckFailed("Could not import: are PyQt4 & PyQt5 both installed?")
+
+            except:
+                # Raise any other exceptions
+                raise
+
+        else:
+            # Multiprocessing OK
+            try:
+                msg = p.map(self.callback, [self])[0]
+            except:
+                # If we hit an error on multiprocessing raise it
+                raise
+            finally:
+                # Tidy up multiprocessing
+                p.close()
+                p.join()
+
+        return msg
+
+
+def backend_qt4_internal_check(self):
+    try:
+        from PyQt4 import QtCore
+    except ImportError:
+        raise CheckFailed("PyQt4 not found")
+
+    try:
+        qt_version = QtCore.QT_VERSION
+        pyqt_version_str = QtCore.QT_VERSION_STR
+    except AttributeError:
+        raise CheckFailed('PyQt4 not correctly imported')
+    else:
         BackendAgg.force = True
-        return ("Qt: %s, PyQt4: %s" %
-                (self.convert_qt_version(
-                    qtconfig.qt_version),
-                    qtconfig.pyqt_version_str))
+        return ("Qt: %s, PyQt: %s" % (self.convert_qt_version(qt_version), pyqt_version_str))
 
 
-class BackendPySide(OptionalBackendPackage):
+class BackendQt4(BackendQtBase):
+    name = "qt4agg"
+
+    def __init__(self, *args, **kwargs):
+        BackendQtBase.__init__(self, *args, **kwargs)
+        self.callback = backend_qt4_internal_check
+
+
+def backend_qt5_internal_check(self):
+    try:
+        from PyQt5 import QtCore
+    except ImportError:
+        raise CheckFailed("PyQt5 not found")
+
+    try:
+        qt_version = QtCore.QT_VERSION
+        pyqt_version_str = QtCore.QT_VERSION_STR
+    except AttributeError:
+        raise CheckFailed('PyQt5 not correctly imported')
+    else:
+        BackendAgg.force = True
+        return ("Qt: %s, PyQt: %s" % (self.convert_qt_version(qt_version), pyqt_version_str))
+
+
+class BackendQt5(BackendQtBase):
+    name = "qt5agg"
+
+    def __init__(self, *args, **kwargs):
+        BackendQtBase.__init__(self, *args, **kwargs)
+        self.callback = backend_qt5_internal_check
+
+
+def backend_pyside_internal_check(self):
+    try:
+        from PySide import __version__
+        from PySide import QtCore
+    except ImportError:
+        raise CheckFailed("PySide not found")
+    else:
+        BackendAgg.force = True
+        return ("Qt: %s, PySide: %s" %
+                (QtCore.__version__, __version__))
+
+
+class BackendPySide(BackendQtBase):
     name = "pyside"
 
-    def check_requirements(self):
-        try:
-            from PySide import __version__
-            from PySide import QtCore
-        except ImportError:
-            raise CheckFailed("PySide not found")
-        else:
-            BackendAgg.force = True
+    def __init__(self, *args, **kwargs):
+        BackendQtBase.__init__(self, *args, **kwargs)
+        self.callback = backend_pyside_internal_check
 
-            return ("Qt: %s, PySide: %s" %
-                    (QtCore.__version__, __version__))
 
 
 class BackendCairo(OptionalBackendPackage):
@@ -1801,11 +2032,16 @@ class BackendCairo(OptionalBackendPackage):
 
     def check_requirements(self):
         try:
-            import cairo
+            import cairocffi
         except ImportError:
-            raise CheckFailed("cairo not found")
+            try:
+                import cairo
+            except ImportError:
+                raise CheckFailed("cairocffi or pycairo not found")
+            else:
+                return "pycairo version %s" % cairo.version
         else:
-            return "version %s" % cairo.version
+            return "cairocffi version %s" % cairocffi.version
 
 
 class DviPng(SetupPackage):
