@@ -32,7 +32,7 @@ public:
     static void init_type();
 
     void draw_bitmap(FT_Bitmap* bitmap, FT_Int x, FT_Int y);
-    void write_bitmap(const char* filename) const;
+    void write_bitmap(FILE* fp) const;
     void draw_rect(unsigned long x0, unsigned long y0,
                    unsigned long x1, unsigned long y1);
     void draw_rect_filled(unsigned long x0, unsigned long y0,
@@ -75,13 +75,17 @@ private:
     unsigned long _height;
 
     void resize(long width, long height);
+
+    // prevent copying
+    FT2Image(const FT2Image&);
+    FT2Image& operator=(const FT2Image&);
 };
 
 class Glyph : public Py::PythonClass<Glyph>
 {
 public:
     Glyph(Py::PythonClassInstance *self, Py::Tuple &args, Py::Dict &kwds) :
-        Py::PythonClass<Glyph>::PythonClass(self, args, kwds) { }
+        Py::PythonClass<Glyph>(self, args, kwds) { }
     virtual ~Glyph();
     static Py::PythonClassObject<Glyph> factory(const FT_Face&, const FT_Glyph&, size_t, long);
     int setattro(const Py::String &name, const Py::Object &value);
@@ -90,6 +94,10 @@ public:
     size_t glyphInd;
 private:
     Py::Dict __dict__;
+
+    // prevent copying
+    Glyph(const Glyph&);
+    Glyph& operator=(const Glyph&);
 };
 
 class FT2Font : public Py::PythonClass<FT2Font>
@@ -133,6 +141,9 @@ private:
     FT_Matrix     matrix;                 /* transformation matrix */
     FT_Vector     pen;                    /* untransformed origin  */
     FT_Error      error;
+    FT_StreamRec  stream;
+    FT_Byte *     mem;
+    size_t        mem_size;
     std::vector<FT_Glyph> glyphs;
     std::vector<FT_Vector> pos;
     double angle;
@@ -143,6 +154,8 @@ private:
     FT_BBox compute_string_bbox();
     void set_scalable_attributes();
 
+    int make_open_args(PyObject *fileobj, FT_Open_Args *open_args);
+
     static char clear__doc__ [];
     static char set_size__doc__ [];
     static char set_charmap__doc__ [];
@@ -167,15 +180,23 @@ private:
     static char get_image__doc__[];
     static char attach_file__doc__[];
     static char get_path__doc__[];
+
+    // prevent copying
+    FT2Font(const FT2Font&);
+    FT2Font& operator=(const FT2Font&);
 };
 
 // the extension module
 class ft2font_module : public Py::ExtensionModule<ft2font_module>
-
 {
 public:
     ft2font_module();
     virtual ~ft2font_module();
+
+private:
+    // prevent copying
+    ft2font_module(const ft2font_module&);
+    ft2font_module operator=(const ft2font_module&);
 };
 
 #endif
