@@ -1,13 +1,18 @@
+"""
+And another broken test...
+"""
+
+from __future__ import print_function
 import sys, time, os
 from matplotlib.ft2font import FT2Font
-from matplotlib.numerix import rand
+from numpy.random import rand
 from matplotlib.backend_bases import GraphicsContextBase
 from matplotlib.backends._backend_agg import RendererAgg
 
 def report_memory(i):
     pid = os.getpid()
     a2 = os.popen('ps -p %d -o rss,sz' % pid).readlines()
-    print i, '  ', a2[1],
+    print(i, '  ', a2[1], end='')
     return int(a2[1].split()[0])
 
 fname = '/usr/local/share/matplotlib/Vera.ttf'
@@ -23,7 +28,7 @@ for i in range(N):
         ys = [400*int(rand()) for k in range(8)]
         rgb = (1,0,0)
         pnts = zip(xs, ys)
-        o.draw_polygon(gc, rgb, pnts)
+        o.draw_polygon(gc, rgb, pnts)   # no such method??
         o.draw_polygon(gc, None, pnts)
 
     for j in range(50):
@@ -48,7 +53,7 @@ for i in range(N):
     if i==1: start = val
 
 end = val
-print 'Average memory consumed per loop: %1.4f\n' % ((end-start)/float(N))
+print('Average memory consumed per loop: %1.4f\n' % ((end-start)/float(N)))
 
 # w/o text and w/o write_png: Average memory consumed per loop: 0.02
 # w/o text and w/ write_png : Average memory consumed per loop: 0.3400
